# -*- coding: utf-8 -*-
#    Copyright (C) 2007 Stewart Adam
#    This file is part of fwbackups.

#    fwbackups is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    fwbackups is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with fwbackups; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
The part of fwbackup that manipulates the actual files.
"""
import os
import os.path
import time
import logging

from fwbackups.i18n import _
from fwbackups.const import *
from fwbackups.config import BackupSetConf, OneTimeConf
from fwbackups import utils, shutil_modded, fwlogger

class EngineError(Exception):
    """ An Engine error """
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class Common:
    """ Common defs for all operations """
    def __init__(self, logger=None):
        # Initialize a logger if none passed
        if not logger:
            self.logger, self.handler = fwlogger._setupLogger(printToo=True)
        else:
            self.logger = logger
        # We're not cancelling...
        self.tocancel = False
        # Totals: None set.
        self.current = 0
        self.total = 0
        # Execute commands with this environment.
        self.environment = {'PATH':os.getenv('PATH')}
        
    def getProgress(self):
        """
        Get the progress of the operation
        Returns: [current#, total#]
        """
        return [self.current, self.total]

    def cancel(self):
        """ Try to cancel the current operation """
        self.tocancel = True
        self.logger.logmsg('INFO', _('Cancel request sent, stopping the current operation...'))

    def onError(self, func, path, exc_info):
        """
        Handles errors when performing file operations.
        func: function called
        path: The path that was being worked on
        exc_info: Good question! I dunno.
        """
        self.logger.logmsg('WARNING', _('Error removing file \'%s\'.') % path)
        self.logger.logmsg('DEBUG', _('Output:\n%s') % ('func:%s\nexc_info:%s' % (func, exc_info)))
        pass

    def ifCancel(self, pid=0):
        """ Check if cancelling the operation is needed. """
        if self.tocancel == True:
            self.logger.logmsg('DEBUG', _('PID recieved: %s') % pid)
            if pid == 0:
                self.logger.logmsg('INFO', _('Operation cancelled.'))
            else:
                utils.kill(pid, 9)
                self.logger.logmsg('INFO', _('Operation cancelled, process #%s killed.') % pid)
            sys.exit(0)
            return False

    def prepDestDir(self, folder):
        """
        Prepare a destination folder
        folder: The folder to prepare
        """
        if not os.path.exists(folder):
            try:
                os.mkdir(folder)
                self.logger.logmsg('DEBUG', _('Creating folder \'%s\'') % folder)
            except:
                self.logger.logmsg('ERROR', _('The folder \'%s\' could not be created.') % folder)
                return False
            # Only raise when something unexpected goes wrong... I think.
            #raise EngineError(_('The backup destination \'%s\' is non-existant and could not be created.') % folder)
        if not utils.CheckPerms(folder) and folder != '/':
            self.logger.logmsg('ERROR', _('You do not have read and write permissions on the folder \'%s\'') % folder)
            return False
            #raise EngineError(_('You do not have read and write permissions on the folder \'%s\'') % folder)
        return True

    def removeIfExists(self, treefile):
        """
        Removes a folder tree or file if it exists already
        treefile: The tree or folder to remove if existing
        """
        # Destination exists?
        self.ifCancel()
        #print 'File: %s || Exists: %s' % (treefile, os.path.exists(treefile))
        if os.path.exists(treefile):
            self.logger.logmsg('WARNING', _('File or folder \'%s\' exists! It will be removed and replaced.') % treefile)
            shutil_modded.rmtree(path=utils.ConvertPath(treefile), onerror=self.onError)



class Restore(Common):
    """ The restore operation """
    def __init__(self, source, dest, logger=None):
        """
        Initializes the restore operation
        source: The source tree or archive to read from
        dest: Folder to write the files to be restored
        logger (optional): The fwbackups logger to use. If none is supplied,
                           a new one will be created.
        """
        Common.__init__(self, logger)
        self.source = source
        self.dest = dest
        self.logger.logmsg('INFO', _('Starting restore operation'))

    def restore(self):
        """ Restores a backup """
        # Set totals
        self.total = 1
        # We don't want to raise a hard error, that's already in the log.
        # So we settle for a simple return false.
        if not self.prepDestDir(self.dest):
            return False

        # Checkpoint 1
        self.ifCancel()

        # Now we set the required commands.
        # tar
        if self.source.endswith('.tar'):
            # chdir so tar works properly.
            if not os.path.isdir(self.source):
                os.chdir(self.dest)
                command = 'cd \'%s\';tar xf \'%s\' -p --same-owner' % (self.dest, self.source)
        # tar.gz
        elif self.source.endswith('.tar.gz'):
            if not os.path.isdir(self.source):
                # chdir so tar works properly.
                os.chdir(self.dest)
                command = 'cd \'%s\';tar xfz \'%s\' -p --same-owner' % (self.dest, self.source)
        # rsync
        else:
            if not os.path.isdir(self.source):
                self.logger.logmsg('ERROR','This source isn\'t a folder or a supported archive! Aborting.')
                return False
            command = 'rsync -g -o -p -r \'%s\'/* \'%s\'' % (self.source, self.dest)
        sub = utils.executeSub(command, env=self.environment, shell=True)
        # Now we're officially working on file 1.
        self.current = 1
        pid = sub.pid
        # Sleep while not done.
        while sub.poll() == "" or sub.poll() == None:
            self.ifCancel(pid)
            time.sleep(5)
        retval = sub.poll()
        # Something go wrong?
        if retval != os.EX_OK:
            self.logger.logmsg('ERROR','Error(s) occurred while restoring certain files or folders.\nPlease check the error output below to determine if any files are incomplete or missing.')
            self.logger.logmsg('ERROR', _('Output:\n%s') % ('Return value: %s\nstdout: %s\nstderr: %s' % (retval, ''.join(sub.stdout.readlines()), ''.join(sub.stderr.readlines()) )))

        # All done!
        self.logger.logmsg('INFO', _('Finished restore operation'))
        return True



class OneTime(Common):
    """ The one-time backup operation """
    def __init__(self, confloc, logger=None):
        """
        Initializes the one-time backup operation
        confloc: location of the configuration
        logger (optional): The fwbackups logger to use. If none is supplied,
                           a new one will be created.
        """
        Common.__init__(self, logger)
        self.confloc = confloc
        self.logger.logmsg('INFO', _('Starting one-time backup operation'))
        # Load configuration and options
        self.set = OneTimeConf()
        self.folderdest = self.set.get('Options', 'Destination')
        self.engine = self.set.get('Options', 'Engine')
        self.recursive = self.set.get('Options', 'Recursive')
        self.pkglist = self.set.get('Options', 'PkgListsToFile')
        self.diskinfo = self.set.get('Options', 'DiskInfoToFile')
        self.dohidden = self.set.get('Options', 'BackupHidden')
        self.excludes = self.set.get('Options', 'Excludes')
        self.nice = int(self.set.get('Options', 'Nice'))
        self.followlinks = self.set.get('Options', 'FollowLinks')
        self.sparse = self.set.get('Options', 'Sparse')
        # Parse backup folder format
        self.dest = utils.ConvertPath('%s/%s-OneTime-%s' % (self.folderdest, _('Backup'), time.strftime('%Y-%m-%d')))

    def backup(self):
        """ One-time backup"""
        # Get the list of paths...
        paths = []
        for i in self.set.options('Paths'):
            # non recursive - Skip dirs
            if not self.recursive == '1' and os.path.isdir(self.set.get('Paths', i)):
                pass
            else:
                paths.append('\'%s\'' % self.set.get('Paths', i))
            #paths.append('\'%s\'' % self.set.get('Paths', i))
            # Set total number of paths
            self.total = self.total + 1

        # Package lists?
        if self.pkglist == '1':
            self.ifCancel()
            utils.execute('rpm -qa --qf "%%{NAME}\n" > "/tmp/%s.txt"' % _('Package list, names only'), env=self.environment, shell=True)
            utils.execute('rpm -qa > "/tmp/%s.txt"' % _('Package list'), env=self.environment, shell=True)
        # Disk info?
        if self.diskinfo == '1':
            self.ifCancel()
            utils.execute('fdisk -l &> "/tmp/%s.txt"' % _('Disk Information'), env=self.environment, shell=True)

        # IF tar || tar.gz, add .tar || tar.gz respectively to the dest since
        # the dest is to be a file, not a folder...
        if self.engine == 'tar':
            self.dest += '.tar'
        if self.engine == 'tar.gz':
            self.dest += '.tar.gz'

        # Now we prep the destination with checkpoints
        self.ifCancel()
        if not self.prepDestDir(self.folderdest):
            return False
        self.ifCancel()
        self.removeIfExists(self.dest)

        self.ifCancel()
        # Commands based on engine
        if self.engine == 'tar':
            command = 'tar rf \'%s\'' % (self.dest)
            os.chdir('/tmp')
            if self.pkglist == '1':
                utils.execute('%s "/tmp/%s.txt" "/tmp/%s.txt"' % (command, _('Package list'), _('Package list, names only')), env=self.environment, shell=True, timeout=-1)
                utils.execute('rm "/tmp/%s.txt" "/tmp/%s.txt" -f' % (_('Package list'), _('Package list, names only')), env=self.environment, shell=True, timeout=-1)
            if self.diskinfo == '1':
                utils.execute('%s "/tmp/%s.txt"' % (command, _('Disk Information')), env=self.environment, shell=True, timeout=-1)
                utils.execute('rm "/tmp/%s.txt" -f' % _('Disk Information'), env=self.environment, shell=True, timeout=-1)
            if not self.recursive == '1':
                command += ' --no-recursion'
            if not self.dohidden == '1':
                command += ' --exclude=.*'
            if self.sparse == '1':
                command += ' -S'
            if self.followlinks == '1':
                command += ' -h'
        # tar.gz
        elif self.engine == 'tar.gz':
            # DON'T rfz - Can't use r (append) and z (gzip) together
            command = 'tar cfz \'%s\'' % (self.dest)
            os.chdir('/tmp')
            if not self.recursive == '1':
                command += ' --no-recursion'
            if not self.dohidden == '1':
                command += ' --exclude=.*'
            if self.sparse == '1':
                command += ' -S'
            if self.followlinks == '1':
                command += ' -h'
        # rsync engine
        elif self.engine == 'rsync':
            os.mkdir(self.dest)
            command = 'rsync -g -o -p -t -R'
            if self.recursive == '1':
                command += ' -r'
            if not self.dohidden == '1':
                command += ' --exclude=.*'
            if self.sparse == '1':
                command += ' -S'
            if self.followlinks == '1':
                command += ' -L'
            else:
                command += ' -l'
            if self.pkglist == '1':
                self.ifCancel()
                utils.execute('%s "/tmp/%s.txt" "/tmp/%s.txt" "%s"' % (command, _('Package list'), _('Package list, names only'), self.dest), env=self.environment, shell=True, timeout=-1)
                utils.execute('rm "/tmp/%s.txt" "/tmp/%s.txt" -f' % (_('Package list'), _('Package list, names only')), env=self.environment, shell=True, timeout=-1)
            if self.diskinfo == '1':
                self.ifCancel()
                utils.execute('%s "/tmp/%s.txt" "%s"' % (command, _('Disk Information'), self.dest), env=self.environment, shell=True, timeout=-1)
                utils.execute('rm "/tmp/%s.txt" -f' % _('Disk Information'), env=self.environment, shell=True, timeout=-1)
        if self.excludes != None and self.excludes != "":
            for i in self.excludes.split('  '):
                command += ' --exclude="%s"' % i

        # Now that the paths & commands are set up...
        # if not gzip, use progress / append
        if not self.dest.endswith('.gz'):
            self.current = 1
            for i in paths:
                # Checkpoint
                self.ifCancel()
                self.logger.logmsg('DEBUG', _('Backing up path %(a)i/%(b)i: %(c)s') % {'a': self.current, 'b': self.total, 'c': i.split('\'')[1]})
                iOrig = i
                if self.engine == 'rsync':
                    self.logger.logmsg('DEBUG', _('Running command: nice -n %(a)i %(b)s %(c)s "%(d)s"' % {'a': self.nice, 'b': command, 'c': i, 'd': self.dest}))
                    sub = utils.executeSub('nice -n %i %s %s "%s"' % (self.nice, command, i, self.dest), env=self.environment, shell=True)
                else:
                    # tar
                    self.logger.logmsg('DEBUG', _('Running command: nice -n %(a)i %(b)s %(c)s' % {'a': self.nice, 'b': command, 'c': i}))
                    sub = utils.executeSub('nice -n %i %s %s' % (self.nice, command, i), env=self.environment, shell=True)
                pid = sub.pid
                self.logger.logmsg('DEBUG', _('Starting subprocess with PID %s') % pid)
                # Sleep while not done.
                while sub.poll() == "" or sub.poll() == None:
                    self.ifCancel(pid)
                    time.sleep(0.01)
                retval = sub.poll()
                self.logger.logmsg('DEBUG', _('Subprocess with PID %(a)s exited with status %(b)s' % {'a': pid, 'b': retval}))
                # Something wrong?
                if retval != os.EX_OK and retval != 2:
                    self.logger.logmsg('ERROR','Error(s) occurred while backing up path \'%s\'.\nPlease check the error output below to determine if any files are incomplete or missing.' % str(i))
                    self.logger.logmsg('ERROR', _('Output:\n%s') % ('Return value: %s\nstdout: %s\nstderr: %s' % (retval, ''.join(sub.stdout.readlines()), ''.join(sub.stderr.readlines()) )))
                self.current = self.current + 1
        # Tar+gzip all at once. No progress.
        else:
            if self.pkglist == '1':
                paths.append('"/tmp/%s.txt" "/tmp/%s.txt"' % (_('Package list'), _('Package list, names only')))
            if self.diskinfo == '1':
                paths.append('"/tmp/%s.txt"' % (_('Disk Information')))
            self.ifCancel()
            # Only 1/1
            self.current = 1
            self.total = 1
            self.logger.logmsg('INFO', _('Using tar.gz: Must backup all paths at once - Progress feature disabled.'))
            self.logger.logmsg('DEBUG', _('Running command: nice -n %(a)i %(b)s %(c)s' % {'a': self.nice, 'b': command, 'c': ' '.join(paths)}))
            sub = utils.executeSub('nice -n %i %s %s' % (self.nice, command, ' '.join(paths)), env=self.environment, shell=True)
            pid = sub.pid
            self.logger.logmsg('DEBUG', _('Starting subprocess with PID %s') % pid)
            # Sleep while not done.
            while sub.poll() == "" or sub.poll() == None:
                self.ifCancel(pid)
                time.sleep(0.01)
            retval = sub.poll()
            self.logger.logmsg('DEBUG', _('Subprocess with PID %(a)s exited with status %(b)s' % {'a': pid, 'b': retval}))
            if self.pkglist == '1':
                utils.execute('rm "/tmp/%s.txt" "/tmp/%s.txt" -f' % (_('Package list'), _('Package list, names only')), env=self.environment, shell=True, timeout=-1)
            if self.diskinfo == '1':
                utils.execute('rm "/tmp/%s.txt" -f' % _('Disk Information'), env=self.environment, shell=True, timeout=-1)

            # Something wrong?
            if retval != os.EX_OK and retval != 2 and retval != None:
                self.logger.logmsg('ERROR','Error(s) occurred while backing up.\nPlease check the error output below to determine if any files are incomplete or missing.' % i)
                self.logger.logmsg('ERROR', _('Output:\n%s') % ('Return value: %s\nstdout: %s\nstderr: %s' % (retval, ''.join(sub.stdout.readlines()), ''.join(sub.stderr.readlines()) )))

        # Backups are private!
        os.chmod(self.dest, 0711)

        # All done!
        self.logger.logmsg('INFO', _('Finished one-time backup'))
        return True



class Automated(Common):
    """ Automated backup operation """
    def __init__(self, setname, logger=None):
        """
        Initializes the automatic backup operation
        setname: Name of the set to automatically backup
        logger (optional): The fwbackups logger to use. If none is supplied,
                           a new one will be created.
        """
        Common.__init__(self, logger)
        self.setname = setname
        self.logger.logmsg('INFO', _('Starting automatic backup operation of set \'%s\'') % self.setname)
        # We're using a set...
        self.set = BackupSetConf(self.setname)
        # Get options
        self.folderdest = self.set.get('Options', 'Destination')
        self.dest = utils.ConvertPath('%s/%s-%s-%s' % (self.folderdest, _('Backup'), self.setname, time.strftime('%Y-%m-%d')))
        self.engine = self.set.get('Options', 'Engine')
        self.command_before = self.set.get('Options', 'CommandBefore')
        self.command_after = self.set.get('Options', 'CommandAfter')
        self.excludes = self.set.get('Options', 'Excludes')
        self.nice = int(self.set.get('Options', 'Nice'))
        if self.set.get('Options', 'Enabled') == 0:
            self.logger.logmsg('INFO', _('Set \'%s\' is not enabled, will not continue') % self.setname)
            return
        self.tokeep = int(float(self.set.get('Options', 'OldToKeep')))
        self.recursive = self.set.get('Options', 'Recursive')
        self.pkglist = self.set.get('Options', 'PkgListsToFile')
        self.diskinfo = self.set.get('Options', 'DiskInfoToFile')
        self.dohidden = self.set.get('Options', 'BackupHidden')
        self.sparse = self.set.get('Options', 'Sparse')
        self.followlinks = self.set.get('Options', 'FollowLinks')

    def backup(self):
        """ Backup a set """
        # Get the list of paths...
        paths = []
        for i in self.set.options('Paths'):
            # non recursive - Skip dirs
            if not self.recursive == '1' and os.path.isdir(self.set.get('Paths', i)):
                pass
            else:
                paths.append('\'%s\'' % self.set.get('Paths', i))
            #paths.append('\'%s\'' % self.set.get('Paths', i))
            # Set total number of paths
            self.total = self.total + 1


        # I come first, let's prepare the destination

        # IF tar || tar.gz, add .tar || tar.gz respectively to the dest since
        # the dest is to be a file, not a folder...
        if self.engine == 'tar':
            self.dest += '.tar'
        if self.engine == 'tar.gz':
            self.dest += '.tar.gz'
        # Prep parent backup dir
        if not self.prepDestDir(self.folderdest):
            return False
        self.removeIfExists(self.dest)

        # Remove old stuff

        # Get list of old backups to remove...
        listing = os.listdir(self.folderdest)
        listing.sort()
        oldbackups = []
        for i in listing:
            if i.startswith('%s-%s-' % (_('Backup'), self.setname)):
                oldbackups.append(i)
        # ...And remove them.
        oldbackups.reverse()
        for i in oldbackups[self.tokeep:]:
            self.ifCancel()
            self.logger.logmsg('DEBUG', _('Removing old backup \'%s\'') % i)
            shutil_modded.rmtree(path=utils.ConvertPath('%s/%s' % (self.folderdest, i)), onerror=self.onError)

        # Before the backup.

        # Before command...
        if self.command_before != None and self.command_before != "":
            self.ifCancel()
            self.logger.logmsg('INFO', _('Executing \'Before\' command'))
            sub = utils.executeSub(self.command_before, env=self.environment, shell=True)
            pid = sub.pid
            self.logger.logmsg('DEBUG', _('Starting subprocess with PID %s') % pid)
            # Sleep while not done.
            while sub.poll() == "" or sub.poll() == None:
                self.ifCancel(pid)
                time.sleep(5)
            retval = sub.poll()
            # Something wrong?
            if retval != os.EX_OK:
                self.logger.logmsg('WARNING', _('Command returned with a non-zero exit status:'))
                self.logger.logmsg('ERROR', 'Return value: %s\nstdout: %s\nstderr: %s' % (retval, ''.join(sub.stdout.readlines()), ''.join(sub.stderr.readlines()) ))
        
        # Package lists?
        if self.pkglist == '1':
            self.ifCancel()
            utils.execute('rpm -qa --qf "%%{NAME}\n" > "/tmp/%s.txt"' % _('Package list, names only'), env=self.environment, shell=True)
            utils.execute('rpm -qa > "/tmp/%s.txt"' % _('Package list'), env=self.environment, shell=True)
        # Disk info?
        if self.diskinfo == '1':
            self.ifCancel()
            utils.execute('fdisk -l &> "/tmp/%s.txt"' % _('Disk Information'), env=self.environment, shell=True)

        self.ifCancel()
        # Commands based on engine
        # tar
        if self.engine == 'tar':
            command = 'tar rf \'%s\'' % (self.dest)
            os.chdir('/tmp')
            if self.pkglist == '1':
                utils.execute('%s "/tmp/%s.txt" "/tmp/%s.txt"' % (command, _('Package list'), _('Package list, names only')), env=self.environment, shell=True, timeout=-1)
                utils.execute('rm "/tmp/%s.txt" "/tmp/%s.txt" -f' % (_('Package list'), _('Package list, names only')), env=self.environment, shell=True, timeout=-1)
            if self.diskinfo == '1':
                utils.execute('%s "/tmp/%s.txt"' % (command, _('Disk Information')), env=self.environment, shell=True, timeout=-1)
                utils.execute('rm "/tmp/%s.txt" -f' % _('Disk Information'), env=self.environment, shell=True, timeout=-1)
            if not self.recursive == '1':
                command += ' --no-recursion'
            if not self.dohidden == '1':
                command += ' --exclude=.*'
            if self.followlinks == '1':
                command += ' -h'
        # tar + gzip
        elif self.engine == 'tar.gz':
            # DON'T rfz - Can't use r (append) and z (gzip) together
            command = 'tar cfz \'%s\'' % (self.dest)
            os.chdir('/tmp')
            if not self.recursive == '1':
                command += ' --no-recursion'
            if not self.dohidden == '1':
                command += ' --exclude=.*'
            if self.followlinks == '1':
                command += ' -h'
        # rsync
        elif self.engine == 'rsync':
            # Create backup dir first
            os.mkdir(self.dest)
            command = 'rsync -l -g -o -p -t -R'
            if self.recursive == '1':
                command += ' -r'
            if not self.dohidden == '1':
                command += ' --exclude=.*'
            if self.pkglist == '1':
                self.ifCancel()
                retval, a, b = utils.execute('%s "/tmp/%s.txt" "/tmp/%s.txt" "%s"' % (command, _('Package list'), _('Package list, names only'), self.dest), env=self.environment, shell=True, timeout=-1)
                utils.execute('rm "/tmp/%s.txt" "/tmp/%s.txt" -f' % (_('Package list'), _('Package list, names only')), env=self.environment, shell=True, timeout=-1)
            if self.diskinfo == '1':
                self.ifCancel()
                utils.execute('%s "/tmp/%s.txt" "%s"' % (command, _('Disk Information'), self.dest), env=self.environment, shell=True, timeout=-1)
                utils.execute('rm "/tmp/%s.txt" -f' % _('Disk Information'), env=self.environment, shell=True, timeout=-1)
            if self.followlinks == '1':
                command += ' -L'
            else:
                command += ' -l'
        if self.excludes != None and self.excludes != "":
            for i in self.excludes.split('  '):
                command += ' --exclude="%s"' % i

        # Now that the paths & commands are set up...
        # if not gzip, use progress / append
        if not self.dest.endswith('.gz'):
            for i in paths:
                self.ifCancel()
                self.current = self.current + 1
                self.logger.logmsg('DEBUG', _('Backing up path \'%s\'') % i.split('\'')[1])
                if self.engine == 'rsync':
                    self.logger.logmsg('DEBUG', _('Running command: nice -n %(a)i %(b)s %(c)s "%(d)s"' % {'a': self.nice, 'b': command, 'c': i, 'd': self.dest}))
                    sub = utils.executeSub('nice -n %i %s %s "%s"' % (self.nice, command, i, self.dest), env=self.environment, shell=True)
                else:
                    # tar
                    self.logger.logmsg('DEBUG', _('Running command: nice -n %(a)i %(b)s %(c)s' % {'a': self.nice, 'b': command, 'c': i}))
                    sub = utils.executeSub('nice -n %i %s %s' % (self.nice, command, i), env=self.environment, shell=True)
                pid = sub.pid
                self.logger.logmsg('DEBUG', _('Starting subprocess with PID %s') % pid)
                # Sleep while not done.
                while sub.poll() == "" or sub.poll() == None:
                    self.ifCancel(pid)
                    time.sleep(0.01)
                retval = sub.poll()
                self.logger.logmsg('DEBUG', _('Subprocess with PID %(a)s exited with status %(b)s' % {'a': pid, 'b': retval}))
                # Something wrong?
                if retval != os.EX_OK and retval != 2 and retval != None:
                    self.logger.logmsg('ERROR','Error(s) occurred while backing up path %s.\nPlease check the error output below to determine if any files are incomplete or missing.' % i)
                    self.logger.logmsg('ERROR', _('Output:\n%s') % ('Return value: %s\nstdout: %s\nstderr: %s' % (retval, ''.join(sub.stdout.readlines()), ''.join(sub.stderr.readlines()) )))
        # Tar+gzip all at once. No progress.
        else:
            self.ifCancel()
            if self.pkglist == '1':
                paths.append('"/tmp/%s.txt" "/tmp/%s.txt"' % (_('Package list'), _('Package list, names only')))
            if self.diskinfo == '1':
                paths.append('"/tmp/%s.txt"' % (_('Disk Information')))
            if not self.recursive == '1':
                command += ' --no-recursion'
            if not self.dohidden == '1':
                command += ' --exclude=.*'
            # Only 1/1
            self.current = 1
            self.total = 1
            self.logger.logmsg('INFO', _('Using tar.gz: Must backup all paths at once - Progress feature disabled.'))
            self.logger.logmsg('DEBUG', _('Running command: nice -n %(a)i %(b)s %(c)s' % {'a': self.nice, 'b': command, 'c': ' '.join(paths)}))
            sub = utils.executeSub('nice -n %i %s %s' % (self.nice, command, ' '.join(paths)), env=self.environment, shell=True)
            pid = sub.pid
            self.logger.logmsg('DEBUG', _('Starting subprocess with PID %s') % pid)
            # Sleep while not done.
            while sub.poll() == "" or sub.poll() == None:
                self.ifCancel(pid)
                time.sleep(0.01)
            retval = sub.poll()
            self.logger.logmsg('DEBUG', _('Subprocess with PID %(a)s exited with status %(b)s' % {'a': pid, 'b': retval}))
            if self.pkglist == '1':
                utils.execute('rm "/tmp/%s.txt" "/tmp/%s.txt" -f' % (_('Package list'), _('Package list, names only')), env=self.environment, shell=True, timeout=-1)
            if self.diskinfo == '1':
                utils.execute('rm "/tmp/%s.txt" -f' % _('Disk Information'), env=self.environment, shell=True, timeout=-1)

            # Something wrong?
            if retval != os.EX_OK and retval != 2 and retval != None:
                self.logger.logmsg('ERROR','Error(s) occurred while backing up.\nPlease check the error output below to determine if any files are incomplete or missing.' % i)
                self.logger.logmsg('ERROR', _('Output:\n%s') % ('Return value: %s\nstdout: %s\nstderr: %s' % (retval, ''.join(sub.stdout.readlines()), ''.join(sub.stderr.readlines()) )))

        # Backups are private!
        os.chmod(self.dest, 0711)

        # After command
        if self.command_after != None and self.command_after != "":
            self.ifCancel()
            self.logger.logmsg('INFO', _('Executing \'After\' command'))
            sub = utils.executeSub(self.command_after, env=self.environment, shell=True)
            pid = sub.pid
            self.logger.logmsg('DEBUG', _('Starting subprocess with PID %s') % pid)
            # Sleep while not done.
            while sub.poll() == "" or sub.poll() == None:
                self.ifCancel(pid)
                time.sleep(5)
            retval = sub.poll()
            # Something wrong?
            if retval != os.EX_OK:
                self.logger.logmsg('WARNING', _('Command returned with a non-zero exit status:'))
                self.logger.logmsg('ERROR', 'Return value: %s\nstdout: %s\nstderr: %s' % (retval, ''.join(sub.stdout.readlines()), ''.join(sub.stderr.readlines()) ))


        # All done!
        self.logger.logmsg('INFO', _('Finished automatic backup operation of set \'%s\'') % self.setname)

