# -*- coding: utf-8 -*-
#    Copyright (C) 2005, 2006, 2007 Stewart Adam
#    This file is part of fwbackups.

#    fwbackups is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    fwbackups is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with fwbackups; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
Core classes for fwbackups.
"""
import logging
import ConfigParser
import sys

from fwbackups.i18n import _
from fwbackups.const import *
from fwbackups import utils

class ConfigError(Exception):
    """ Errors in the configuration file. """
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

def _setupConf():
    """ Setup the configuration directory """
    for i in [loc, setloc]:
        if not os.path.exists(i):
            try:
                os.mkdir(i)
            except OSError:
                print _('ERROR: Error creating the configuration directory. Aborting.')
                sys.exit(1)
        if not utils.CheckPerms(i):
            print _('ERROR: You do not have read and write permissions to modify \'%s\'. Aborting.') % i
            sys.exit(1)

class ConfigFile(ConfigParser.ConfigParser):
    """ A more sane ConfigParser. It commits changes immediately,
        and also re-parses after each action so
        what-you-see-is-what's-in-the-file.
    """
    # Basic setup.
    def __init__(self, conffile, create=False, logger=None):
        ConfigParser.ConfigParser.__init__(self)
        self.config = ConfigParser.ConfigParser()
        self.conffile = conffile
        self.create = create
        if not logger:
            self.logger = logging.getLogger(LOGGERNAME)
        else:
            self.logger = logger
        # Finally...
        self.read()

    def set_conffile(self, conffile, create=True):
        """ Use a new config file.
            conffile: the path to the new configuration file
            create: If it doesn't exist, create a new config?
        """
        self.conffile = conffile
        self.read()

    def get_conffile(self):
        """ Returns the config file path. """
        return self.conffile

    def read(self):
        """ Read & parse the config file."""
        self.logger.logmsg('DEBUG', _('Reading configuration file \'%s\'') % self.conffile)
        if self.create == True and not os.path.exists(self.conffile):
            self.logger.logmsg('DEBUG', _('Creating new configuration file \'%s\'') % self.conffile)
            self.fh = open(self.conffile,'w')
            self.fh.write('')
            self.fh.close()
        elif self.create == False and not os.path.exists(self.conffile):
            raise ConfigError(_('Attempted to read non-existant file \'%s\'') % self.conffile)
        self.fh = open(self.conffile,'r')
        self.config.readfp(self.fh)
        self.fh.close()

    def write(self):
        """ Write the config file, then reparse. """
        try:
            self.logger.logmsg('DEBUG', _('Writing configuration file \'%s\'') % self.conffile)
            self.fh = open(self.conffile,'w')
            self.config.write(self.fh)
            self.fh.close()
            self.read()
        except:
            raise ConfigError(_('Cannot write to file \'%s\'') % self.conffile)

    def set(self, section, prop, value):
        """ Set a value in a given section and save. """
        self.config.set(section, prop, value)
        self.write()
        return True

    def get(self, section, prop):
        """ Returns the value for a given option in a section """
        value = self.config.get(section, prop)
        return value

    def remove_option(self, sectname, optionname):
        """ Remove a property & it's value, then save. """
        self.config.remove_option(sectname, optionname)
        self.write()
        return True

    def add_section(self, sectname):
        """ Add a section and save. """
        self.config.add_section(sectname)
        self.write()
        return True

    def remove_section(self, sectname):
        """ Remove a section and save. """
        self.config.remove_section(sectname)
        self.write()
        return True

    def sections(self):
        """ Returns the sections in the file """
        return self.config.sections()

    def options(self, section):
        """ Returns the options in a section of the file """
        return self.config.options(section)

    def has_section(self, section):
        """ Has section x? """
        return self.config.has_section(section)

    def has_option(self, section, option):
        """ Has option x in section y? """
        return self.config.has_option(section, option)

class BackupSetConf(ConfigFile):
    """ Backup set configuration file """
    def __init__(self, setName, create=False, logger=None):
        """ Checks for previous versions, incomplete confs, etc """
        self.setName = setName
        ConfigFile.__init__(self, utils.ConvertPath('%s/%s.conf' % (setloc, setName)), create, logger)
        if self.config.sections() == [] and create == True:
            self._setupBasicConf()

        for i in ['General', 'Paths', 'Options', 'Times']:
            if not self.config.has_section(i):
                raise ConfigError(_('Set \'%(a)s\' is missing section \'%(b)s\'!') % {'a': setName, 'b': i})

        if self.config.get('General', 'Version') != VERSION:
            self._import()
        if self.config.get('General', 'Type') != 'Set':
            raise ConfigError(_('\'%(a)s\' isn\'t a set configuration file: Its type is set to \'%(b)s\'.') %\
                {'a': self.setName, 'b': self.config.get('General', 'Type')})
        # I had changed the cron format in the .conf file in beta2/3? Don't remember
    def _setupBasicConf(self):
        """ Creates a basic set configuration file """
        self.config.add_section('General')
        self.config.set('General', 'Version', VERSION)
        self.config.set('General', 'Type', 'Set')
        self.config.add_section('Paths')
        self.config.add_section('Options')
        self.config.add_section('Times')
        self.config.set('Times', 'Custom','False')
        self.config.set('Times', 'Entry', '0 0 * * *')
        self.config.set('Options', 'Enabled', 1)
        self.config.set('Options', 'CommandBefore', '')
        self.config.set('Options', 'CommandAfter', '')
        self.config.set('Options', 'OldToKeep', 1)
        self.config.set('Options', 'Destination', userhome)
        self.config.set('Options', 'Recursive', 1)
        self.config.set('Options', 'PkgListsToFile', 1)
        self.config.set('Options', 'DiskInfoToFile', 0)
        self.config.set('Options', 'BackupHidden', 1)
        self.config.set('Options', 'Engine', 'tar')
        self.config.set('Options', 'Sparse', 0)
        self.config.set('Options', 'Nice', 0)
        self.config.set('Options', 'Excludes', '')
        self.config.set('Options', 'FollowLinks', '0')
        self.write()
        self.logger.logmsg('DEBUG', _('Setting defaults in configuration file for set \'%s\'') % self.setName)

    def _import(self):
        """ Import old configurations. Only runs if not current version """
        oldVersion = self.config.get('General', 'Version')
        self.logger.logmsg('INFO', _('Converting set \'%(a)s\' of version %(b)s to the current version (v%(c)s)') % \
            {'a': self.setName, 'b': oldVersion, 'c': VERSION})

        # only if it's a version mismatch should we import
        if oldVersion == VERSION:
            return self.set('General', 'Version', VERSION)

        # This lets us do what needs to be done for version X and _above_
        fromHereUp = False

        # --------------------------------------------------------
        # oldest first, newest last in for the order of IF clauses
        # --------------------------------------------------------

        # I had changed the cron format in the .conf file in beta2/3? Don't remember
        if oldVersion == '1.43' or oldVersion.startswith('1.43.0') or fromHereUp == True:
            fromHereUp = True
            cron = self.config.get('Times', 'Entry').split(' ')
            tempCron = cron[:5]
            tempCron.append('fwbackups-run')
            self.set('Times', 'Entry', ' '.join(tempCron))

        # I added sparse & nice & excludes in 1.43.0rc3
        if oldVersion.startswith('1.43.0rc2') or fromHereUp == True:
            fromHereUp = True
            self.set('Options', 'Excludes', '')
            self.set('Options', 'Nice', 0)
            self.set('Options', 'Sparse', 0)

        # I added follow links in 1.43.1
        if oldVersion.startswith('1.43.1') or fromHereUp == True:
            fromHereUp = True
            self.set('Options', 'FollowLinks', '0')
        return self.set('General', 'Version', VERSION)

class OneTimeConf(ConfigFile):
    """ One-Time backup configuration file """
    def __init__(self, create=False, logger=None):
        """ Checks for previous versions, incomplete confs, etc """
        if create == True and os.path.exists(onetimeloc):
            os.remove(onetimeloc)
        ConfigFile.__init__(self, onetimeloc, create, logger)
        if self.config.sections() == [] and create == True:
            self._setupBasicConf()
        for i in ['General', 'Paths', 'Options']:
            if not self.config.has_section(i):
                raise ConfigError(_('Set \'%(a)s\' is missing section \'%(b)s\'!' % {'a': onetimeloc, 'b': i}))
        if self.config.get('General', 'Type') != 'OneTime':
            raise ConfigError(_('\'%(a)s\' isn\'t a one time configuration file: Its type is set to \'%(b)s\'.') % \
                {'a': onetimeloc, 'b': self.config.get('General', 'Type')})

    def _setupBasicConf(self):
        """ Creates a basic onetime configuration file """
        self.config.add_section('General')
        self.config.set('General', 'Type', 'OneTime')
        self.config.add_section('Paths')
        self.config.add_section('Options')
        self.config.set('Options', 'Destination', userhome)
        self.config.set('Options', 'Recursive', 1)
        self.config.set('Options', 'PkgListsToFile', 1)
        self.config.set('Options', 'DiskInfoToFile', 0)
        self.config.set('Options', 'BackupHidden', 1)
        self.config.set('Options', 'Engine', 'tar')
        self.config.set('Options', 'Sparse', '0')
        self.config.set('Options', 'Nice', '0')
        self.config.set('Options', 'Excludes', '')
        self.config.set('Options', 'FollowLinks', '0')
        self.write()

class PrefsConf(ConfigFile):
    """ Preferences configuration file """
    def __init__(self, create=False, logger=None):
        """ Checks for previous versions, incomplete confs, etc """
        ConfigFile.__init__(self, prefsloc, create, logger)
        if self.config.sections() == [] and create == True:
            self._setupBasicConf()

        for i in ['General', 'Preferences']:
            if not self.config.has_section(i):
                raise ConfigError(_('Preference configuration is missing section \'%(a)s\'!') % {'a': i})

        if self.config.get('General', 'Version') != VERSION:
            self._import()
        if self.config.get('General', 'Type') != 'Preferences':
            raise ConfigError(_('\'%(a)s\' isn\'t a preference configuration file: Its type is set to \'%(b)s\'.') %\
                {'a': prefsloc, 'b': self.config.get('General', 'Type')})

    def _setupBasicConf(self):
        """ Creates a basic set configuration file """
        self.config.add_section('General')
        self.config.set('General', 'Version', VERSION)
        self.config.set('General', 'Type', 'Preferences')
        self.config.add_section('Preferences')
        self.config.set('Preferences', 'ShowTrayIcon', 1)
        self.config.set('Preferences', 'ShowNotifications', 1)
        self.config.set('Preferences', 'DontShowMe_OldVerWarn', 0)
        self.config.set('Preferences', 'DontShowMe_ClearLog', 0)
        self.write()
        self.logger.logmsg('DEBUG', _('Setting defaults in preferences configuration file'))

    def _import(self):
        """ Import old configurations. Only runs if not current version """
        oldVersion = self.config.get('General', 'Version')
        self.logger.logmsg('INFO', _('Converting preferences from version %(a)s to the current version (v%(b)s)') % \
            {'a': oldVersion, 'b': VERSION})

        # only if it's a version mismatch should we import
        if oldVersion == VERSION:
            return self.set('General', 'Version', VERSION)

        # This lets us do what needs to be done for version X and _above_
        fromHereUp = False

        """# I had changed the cron format in the .conf file in beta2/3? Don't remember
        if oldVersion == '1.43' or oldVersion.startswith('1.43.0') or fromHereUp == True:
            fromHereUp = True
            cron = self.config.get('Times', 'Entry').split(' ')
            tempCron = cron[:5]
            tempCron.append('fwbackups-run')
            self.set('Times', 'Entry', ' '.join(tempCron))

        # I added sparse & nice & excludes in 1.43.0rc3
        if oldVersion.startswith('1.43.0rc') or fromHereUp == True:
            fromHereUp = True
            self.set('Options', 'Sparse', 0)
            self.set('Options', 'Nice', 0)
            self.set('Options', 'Excludes', '')
            self.set('General', 'Version', VERSION)"""
        
        return self.set('General', 'Version', VERSION)

