# -*- coding: utf-8 -*-
#    Copyright (C) 2005, 2006, 2007 Stewart Adam
#    This file is part of fwbackups.

#    fwbackups is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    fwbackups is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with fwbackups; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
The callbacks for everything Glade.
"""
import gtk
import gtk.glade
import gobject
import logging
import webbrowser
import time

from i18n import _
from const import *
import utils, config, guiutils, backend, shutil_modded, cron
from guiwidgets import ErrorDia, OptionsDia, ConfirmDia, SaveDia,\
ProgressBar, ExportDia, InfoDia, PathDia, PreferenceDia

class UI(gtk.glade.XML):
    """Base class for UIs loaded from glade."""
    def __init__(self, filename, rootname,domain=None):
        """
            Initialize a new instance.
            `filename' is the name of the .glade file containing the UI hierarchy.
            `rootname' is the name of the topmost widget to be loaded.
            `gladeDir' is the name of the directory, relative to the Python
            path, in which to search for `filename'
        """
        if domain:
            gtk.glade.XML.__init__(self, filename, root=None, domain=domain)
        else:
            gtk.glade.XML.__init__(self, filename, root=None, domain=None)
        self.filename = filename
        self.root = self.get_widget(rootname)

    def __getattr__(self, name):
        """Look up an as-yet undefined attribute, assuming it's a widget."""
        result = self.get_widget(name)
        if result is None:
            raise AttributeError("Can't find widget %s in %s.\n" %
                                 (`name`, `self.filename`))
        
        # Cache the widget to speed up future lookups.  If multiple
        # widgets in a hierarchy have the same name, the lookup
        # behavior is non-deterministic just as for libglade.
        setattr(self, name, result)
        return result

class Controller:
    """ Base class for all controllers of glade-derived UIs. """
    def __init__(self, ui):
        """ Initialize a new instance.
            `ui' is the user interface to be controlled. """
        self.ui = ui
        self.ui.signal_autoconnect(self._getAllMethods())

    def _getAllMethods(self):
        """ Get a dictionary of all methods in self's class hierarchy. """
        result = {}

        # Find all callable instance/class attributes.  This will miss
        # attributes which are "interpreted" via __getattr__.  By
        # convention such attributes should be listed in
        # self.__methods__.
        allAttrNames = self.__dict__.keys() + self._getAllClassAttributes()
        for name in allAttrNames:
            value = getattr(self, name)
            if callable(value):
                result[name] = value
        return result

    def _getAllClassAttributes(self):
        """ Get a list of all attribute names in self's class hierarchy. """
        nameSet = {}
        for currClass in self._getAllClasses():
            nameSet.update(currClass.__dict__)
        result = nameSet.keys()
        return result

    def _getAllClasses(self):
        """ Get all classes in self's heritage. """
        result = [self.__class__]
        i = 0
        while i < len(result):
            currClass = result[i]
            result.extend(list(currClass.__bases__))
            i = i + 1
        return result


class Callbacks( Controller ):
    def __init__( self ):
        # Create and ui object contains the widgets.
        ui = UI(GLADE_FILE, 'main', 'fwbackups' )
        # init the Controller Class to connect signals.
        Controller.__init__(self, ui)
        self.lock = False
        self.ui.trayNotify = self.trayNotify
        try:
            import gnome
        except ImportError:
            self.logger.logmsg('ERROR', _('Cannot locate the gnome bindings for\
 python. Will use to online documentation instead.'))

    ###
    ### TRAY ICON ###
    ###

    def on_show_administrator1_activate(self, widget, event=None):
        """ Hide/show the main window """
        if self.ui.show_administrator1.get_active():
            self.ui.main.show()
        else:
            self.ui.main.hide()

    def on_hide1_activate(self, widget, event=None):
        """ Hides the tray icon until it displays a notification again """
        prefs = config.PrefsConf()
        prefs.set('Preferences', 'ShowTrayIcon', 0)
        if not self.ui.main.get_property('visible'):
            self.ui.main.show()
        self.getTrayIcon().set_visible(False)

    ###
    ### WRAPPERS ###
    ###
    def help(self):
        """ Display help  """
        try:
            import gnome
            gnome.help_display('fwbackups')
        except ImportError:
            webbrowser.open_new('http://www.diffingo.com/downloads/fwbackups/docs/html')
        except Exception, error:
            self.logger.logmsg('WARNING', _('Unexpected error \'%s\' while trying \
to show documentation!\nUsing online documentation instead.' % str(error)))
            webbrowser.open_new('http://www.diffingo.com/downloads/fwbackups/docs/html')

    def hide(self, widget, event=None):
        """ Wrapper for closing a window non-destructively """
        widget.hide()

    def main_close(self, widget, event=None):
        """ Wrapper for quitting """
        self.SetView1.regenerateCronTab()
        if self.operationInProgress == True:
            confirmDia = ConfirmDia(\
                                    self.ui.confirm_dia,\
                                    self.ui.ConfirmDiaLabel,\
                                    _('An operation is in progress. Are you sure you want to quit?')\
                                    )
            response = confirmDia.run()
            confirmDia.destroy()
            if response == gtk.RESPONSE_NO:
                return True
        if gtk.main_level() > 1:
            gtk.main_quit()
            gobject.idle_add(self.main_close,100)
        else:
            # do normal stuff I do when the app quits...
            if self.logger:
                self.logger.logmsg('INFO', _('fwbackups administrator closed'))
                logging.shutdown()
            try:
                gtk.main_quit()
            except RuntimeError, msg:
                print 'gtk.main_quit() RuntimeError: %s' % msg
                pass
        return False

    ###
    ### MENUS ###
    ###

    ### FILE MENU
    def on_new_set1_activate(self, widget):
        """ New Set entry in menu """
        self.SetView1.new()
    def on_import_sets1_activate(self, widget):
        """ Import Set entry in menu """
        fileDialog = PathDia(self.ui.path_dia, gtk.FILE_CHOOSER_ACTION_OPEN,\
                              ffilter=['*.conf','Configuration files (*.conf)'],\
                              multiple=False, )
        response = fileDialog.run()
        if response == gtk.RESPONSE_OK:
            oldsetname = fileDialog.get_filenames()[0]
            if os.path.exists(utils.ConvertPath('%s/%s.conf' % (setloc, 'imported'))):
                errorDia = ErrorDia(self.ui.error_dia, self.ui.ErrorDiaLabel,\
                                    _('The set \'imported\' already exists. Please rename it first.'))
                errorDia.run()
                errorDia.destroy()
                fileDialog.destroy()
                return
            else:
                oldset = config.ConfigFile(oldsetname)
                if 'General' not in oldset.sections():
                    # It's created automatically, so no need to copy.
                    set = config.BackupSetConf('imported', True)
                    # old fwbackups config...
                    self.SetView1.pathnumber = 0
                    for i in oldset.options('paths'):
                        self.SetView1._saveRow(None, None, None, set,\
                                               oldset.get('paths', i))
                    if oldset.get('prefs', 'autotar'):
                        set.set('Paths', 'path%s' %\
                               (self.SetView1.pathnumber + 1), '/etc')
                    set.set('Options', 'PkgListsToFile',\
                            oldset.get('prefs', 'autorpm'))
                    set.set('Options', 'OldToKeep',\
                            oldset.get('prefs', 'tokeep'))
                    set.set('Options', 'DiskInfoToFile',\
                            oldset.get('prefs', 'autodiskinfo'))
                else:
                    # In this case, we need to copy.
                    shutil_modded.copy(oldset.conffile, '%s/imported.conf' % setloc)
                    set = config.BackupSetConf('imported')
                    set._import()
            l = set.get('Times', 'Entry').split(' ')
            # name automatically dropped by _import() so adding it at the end
            l.append("'imported'")
            set.set('Times', 'Entry', ' '.join(l))
            cronh = cron.CronTab()
            cronh.environ = {'EDITOR':'%s/cronwriter.py' % MAIN_PATH}
            if not cronh.search(l):
                cronh.add(l)
        fileDialog.destroy()
        self.SetView1.refresh()
    
    def on_backup_set_now1_activate(self, widget):
        """ Backup the selected set """
        self.SetView1.startBackup()

    def on_restore_set1_activate(self, widget):
        """ Restore folder or archive """
        self.ui.MainNotebook.set_current_page(3)
        self.ui.RestoreSourceFolderRadio.set_active(True)
        self.ui.RestoreRadioTool.set_active(True)
    
    def on_restore_set2_activate(self, widget):
        """ Restore a set """
        self.ui.MainNotebook.set_current_page(3)
        self.ui.RestoreSourceSetRadio.set_active(True)
        self.ui.RestoreRadioTool.set_active(True)
    
    
    def _exportSet(self, model, path, iter, dest):
        """ Exports a set. Arguments are standard for a liststore.foreach,
            except dest...
            dest: The destination to export to
        """
        if self.ExportView1.liststore.get_value(iter, 0):
            shutil_modded.copy('%s/%s.conf' % (setloc,\
self.ExportView1.liststore.get_value(iter, 1)), dest)

    def on_export_sets1_activate(self, widget):
        """ Export Sets entry in menu """
        self.ExportView1.refresh()
        exportDialog = ExportDia(self.ui.export_dia)
        response = exportDialog.run()
        if response == gtk.RESPONSE_HELP:
            self.help()
            response = exportDialog.run()
        if response == gtk.RESPONSE_OK:
            destination = self.ui.ExportFileChooserButton.get_filename()
            self.ExportView1.liststore.foreach(self._exportSet, destination)
            infoDialog = InfoDia(self.ui.info_dia, self.ui.InfoDiaLabel,\
                                 _('The selected sets were exported successfully.'))
            infoDialog.run()
            infoDialog.destroy()
        exportDialog.destroy()

    def on_quit1_activate(self, widget):
        """ Quit entry in menu """
        self.main_close(self)
        return False

    ### EDIT MENU
    def on_edit_set1_activate(self, widget):
        """ Edit Set entry in menu """
        self.SetView1.edit()

    def on_remove_set1_activate(self, widget):
        """ Remove Set entry in menu """
        self.SetView1.remove()

    def on_duplicate_set1_activate(self, widget):
        """ Duplicate Set entry in menu """
        try:
            selected = guiutils.getSelectedItems(self.ui.BackupSetIconview)[0]
            iterator = self.SetView1.liststore.get_iter(selected)
            setname = self.SetView1.liststore.get_value(iterator, 0)
        except IndexError:
            self.statusbar.newmessage(_('Please select a set before choosing an action.'), 3)
            return
        set = config.BackupSetConf(setname)
        newsetname = setname
        l = set.get('Times', 'Entry').split(' ')
        cronh = cron.CronTab()
        cronh.environ = {'EDITOR':'%s/cronwriter.py' % MAIN_PATH}
        newsetname += '_copy'
        newsetpath = utils.ConvertPath('%s/%s.conf' % (setloc, newsetname))
        t = l[-1].split("'")
        t[0] += '_copy'
        l[-1] = "'".join(t)
        while os.path.exists(newsetpath):
            newsetname += '_copy'
            newsetpath = utils.ConvertPath('%s/%s.conf' % (setloc, newsetname))
            t = l[-1].split("'")
            t[0] += '_copy'
            l[-1] = "'".join(t)
        shutil_modded.copy(utils.ConvertPath('%s/%s.conf' % (setloc, setname)),\
                           newsetpath)
        set = config.BackupSetConf('%s' % newsetname)
        set.set('Times', 'Entry', ' '.join(l))
        cronh.add(l)
        self.SetView1.refresh()

    ### VIEW MENU

    # Switch tabs...    # Switch tabs...    # Switch tabs...
    """ It's the same thing over and over so I'll explain here.
        There are the menu callbacks under the View menu to switch to the 
        various tabs. Each function is essentially  doing 'When I'm toggled,
        check if I'm sensitive.
         > If yes, switch me to the correct tab.
         > If no, do nothing as I shouldn't be switching!
    """
    def on_overview1_activate(self, widget):
        if self.ui.OverviewRadioTool.get_property('sensitive') == 1:
            self.ui.MainNotebook.set_current_page(0)
            self.ui.OverviewRadioTool.set_active(True)
            return True
        else:
            return False
    def on_user_backup_sets1_activate(self, widget):
        if self.ui.BackupSetsRadioTool.get_property('sensitive') == 1:
            self.ui.MainNotebook.set_current_page(1)
            self.ui.BackupSetsRadioTool.set_active(True)
            return True
        else:
            return False
    def on_one_time_backup1_activate(self, widget):
        if self.ui.OneTimeRadioTool.get_property('sensitive') == 1:
            self.ui.MainNotebook.set_current_page(2)
            self.ui.OneTimeRadioTool.set_active(True)
            return True
        else:
            return False
    def on_restore1_activate(self, widget):
        if self.ui.RestoreRadioTool.get_property('sensitive') == 1:
            self.ui.MainNotebook.set_current_page(3)
            self.ui.RestoreRadioTool.set_active(True)
            return True
        else:
            return False
    def on_log1_activate(self, widget):
        if self.ui.LogViewRadioTool.get_property('sensitive') == 1:
            self.ui.MainNotebook.set_current_page(4)
            self.ui.LogViewRadioTool.set_active(True)
            return True
        else:
            return False
    """ It's the same thing over and over so I'll explain here.
        There are the toolbar button callbacks to switch to the various tabs.
    """
    def on_OverviewRadioTool_clicked(self, widget):
        self.ui.MainNotebook.set_current_page(0)
    def on_BackupSetsRadioTool_clicked(self, widget):
        self.ui.MainNotebook.set_current_page(1)
    def on_OneTimeRadioTool_clicked(self, widget):
        self.ui.MainNotebook.set_current_page(2)
    def on_RestoreRadioTool_clicked(self, widget):
        self.ui.MainNotebook.set_current_page(3)
    def on_LogViewRadioTool_clicked(self, widget):
        self.ui.MainNotebook.set_current_page(4)
    # /Switch tabs...    # /Switch tabs...    # /Switch tabs...

    def on_preferences1_activate(self, widget, event=None):
        """ Shows preferences """
        prefs = config.PrefsConf(create=True, logger=self.logger)
        preferences_dia = PreferenceDia(self.ui.preferences_dia)
        if int(prefs.get('Preferences', 'ShowTrayIcon')) == 1:
            self.ui.preferencesShowTrayIconCheck.set_active(True)
        else:
            self.ui.preferencesShowTrayIconCheck.set_active(False)
        if int(prefs.get('Preferences', 'ShowNotifications')) == 1:
            self.ui.preferencesShowNotificationsCheck.set_active(True)
        else:
            self.ui.preferencesShowNotificationsCheck.set_active(False)
        response = preferences_dia.run()
        if response == gtk.RESPONSE_HELP:
            self.help()
            response = preferences_dia.run()
        preferences_dia.destroy()


    ### HELP MENU
    def on_about1_activate(self, widget):
        """ Help menu entry """
        self.ui.about.show()

    def on_help1_activate(self, widget):
        """ Help menu entry """
        self.help()

    ###
    ### PREFERENCES DIALOG ###
    ###

    def on_preferencesShowTrayIconCheck_toggled(self, widget):
        """ Show tray icon check """
        prefs = config.PrefsConf(create=True, logger=self.logger)
        # reparse because of reset button making other mods
        prefs.read()
        # Tray icon
        if self.ui.preferencesShowTrayIconCheck.get_active():
            prefs.set('Preferences', 'ShowTrayIcon', 1)
            self.getTrayIcon().set_visible(True)
        else:
            prefs.set('Preferences', 'ShowTrayIcon', 0)
            self.getTrayIcon().set_visible(False)

    def on_preferencesShowNotificationsCheck_toggled(self, widget):
        """ Show notifications check """
        prefs = config.PrefsConf(create=True, logger=self.logger)
        # reparse because of reset button making other mods
        prefs.read()
        # Notifications
        if self.ui.preferencesShowNotificationsCheck.get_active():
            prefs.set('Preferences', 'ShowNotifications', 1)
        else:
            prefs.set('Preferences', 'ShowNotifications', 0)

    def on_preferencesResetDontShowMeButton_clicked(self, widget, event=None):
        """ Resets all "Don't show me again" messages """
        prefs = config.PrefsConf(logger=self.logger)
        for i in prefs.options('Preferences'):
            if i.startswith('dontshowme_'):
                prefs.set('Preferences', i, 0)
                self.logger.logmsg('DEBUG', _('Resetting preference \'%(a)s\' to 0' % {'a' : i}))

    ###
    ### ABOUT WINDOW ###
    ###
    def on_AboutCloseButton_clicked(self, widget):
        """ Close button for About """
        self.hide(self.ui.about)
        
    def on_AboutDiffingoComButton_clicked(self, widget):
        webbrowser.open_new('http://www.diffingo.com/opensource')

    ###
    ### BACKUPSET_DIA WINDOW ###
    ###
    def on_SetDestination_current_folder_changed(self, widget):
        """ Called when the set destination's folder changes.
            Check the permissions when the set destination change. """
        self.SetView1._checkDestPerms(widget.get_filename(), self.ui.SetDestinationPermissionImage)

    def on_OneTimeDestination_current_folder_changed(self, widget):
        """ Called when the one-time destination's folder changes.
            Checks the permissions when the onetime destination changed. """
        self.OneTimeView1._checkDestPerms(widget.get_filename(), self.ui.OneTimeDestinationPermissionImage)

    ### TAB 1: PATHS
    def on_BackupDiaTreeview_row_activated(self, widget, path, column):
        """ Double click on BackupDiaTreeview """
        self.SetEditorPathView1.editPath()

    def on_BackupSetAddFolderButton_clicked(self, widget):
        """ Add path to backupset_dia """
        self.SetEditorPathView1.addFolder()
    def on_BackupSetAddFileButton_clicked(self, widget):
        """ Add file """
        self.SetEditorPathView1.addFile()
    def on_BackupSetEditPathButton_clicked(self, widget):
        """ Edit path """
        self.SetEditorPathView1.editPath()
    def on_BackupSetRemovePathButton_clicked(self, widget):
        """ Remove path """
        self.SetEditorPathView1.removePath()

    ### TAB 2: TIMES
    # Time: Only/At
    def on_MonthOnlyBox_changed(self, widget):
        """ Box changed; Set the right radiobutton """
        self.ui.MonthOnly.set_active(True)
    def on_DaysWeekOnlyBox_changed(self, widget):
        """ Box changed; Set the right radiobutton """
        self.ui.DaysWeekOnly.set_active(True)
    def on_HoursAtBox_changed(self, widget):
        """ Box changed; Set the right radiobutton """
        self.ui.HoursAt.set_active(True)
    def on_MinutesOnlyScale_change_value(self, widget, scroll, value):
        """ Scale changed; Set the right radiobutton """
        self.ui.MinutesAt.set_active(True)
    # Time: Range (1)
    def on_MonthFromBox1_changed(self, widget):
        """ Box changed; Set the right radiobutton """
        self.ui.MonthFrom.set_active(True)
    def on_DaysWeekFromBox1_changed(self, widget):
        """ Box changed; Set the right radiobutton """
        self.ui.DaysWeekFrom.set_active(True)
    # Time: Range (2)
    def on_MonthFromBox2_changed(self, widget):
        """ Box changed; Set the right radiobutton """
        self.ui.MonthFrom.set_active(True)
    def on_DaysWeekFromBox2_changed(self, widget):
        """ Box changed; Set the right radiobutton """
        self.ui.DaysWeekFrom.set_active(True)

    def on_EasyConfigExpander_activate(self, widget):
        """ Easy time config toggled: Reverse expanders """
        self.ui.EasyConfigTable.set_sensitive(not self.ui.EasyConfigTable.get_property('sensitive'))
        self.ui.ManualConfigTable.set_sensitive(not self.ui.ManualConfigTable.get_property('sensitive'))
        time.sleep(0.2)
        self.ui.ManualConfigExpander.set_expanded(not self.ui.ManualConfigExpander.get_expanded())

    def on_ManualConfigExpander_activate(self, widget):
        """ Manual time config toggled: Reverse expanders """
        self.ui.EasyConfigTable.set_sensitive(not self.ui.EasyConfigTable.get_property('sensitive'))
        self.ui.ManualConfigTable.set_sensitive(not self.ui.ManualConfigTable.get_property('sensitive'))
        time.sleep(0.2)
        self.ui.EasyConfigExpander.set_expanded(not self.ui.EasyConfigExpander.get_expanded())

    ###
    ### MAIN WINDOW ###
    ###
    ### TAB 1: OVERVIEW
    # Handled by runapp, set_labels
    ### TAB 2: SETS
    def on_BackupSetIconview_item_activated(self, widget, event):
        """ Double click on BackupSetIconview """
        self.SetView1.edit()

    def on_OneTimeTreeview_row_activated(self, widget, path, column):
        """ Double click on OneTimeTreeview """
        self.OneTimeView1.editPath()

    def on_NewSetButton_clicked(self, widget):
        """ New Set button in main """
        self.SetView1.new()

    def on_EditSetButton_clicked(self, widget):
        """ Edit Set button in main """
        self.SetView1.edit()

    def on_RemoveSetButton_clicked(self, widget):
        """ Remove Set button in main """
        self.SetView1.remove()

    def on_BackupSetNowButton_clicked(self, widget):
        """ Backup now button in main """
        self.SetView1.startBackup()

    def on_BackupSetCancelButton_clicked(self, widget):
        """ Cancel set backup button in main """
        self.SetView1.cancelBackup()
        
    def on_RestoreSetToButton_clicked(self, widget):
        """ Restore button in main """
        try:
            selected = self.ui.BackupSetIconview.get_selected_items()[0]
            iterator = self.SetView1.liststore.get_iter(selected)
            setname = self.SetView1.liststore.get_value(iterator, 0)
        except IndexError:
            self.statusbar.newmessage(_('Please select a set before choosing an action.'), 3)
            return
        self.ui.MainNotebook.set_current_page(3)
        self.ui.RestoreRadioTool.set_active(True)
        self._setComboboxFromName(setname)

    ### TAB 3: ONE-TIME BACKUP
    def on_OneTimeStartButton_clicked(self, widget):
        """ Start one-time backup on Main """
        self.OneTimeView1.startBackup()
    def on_OneTimeCancelButton_clicked(self, widget):
        """ Cancel one-time backup on Main """
        self.OneTimeView1.cancelBackup()
    def on_OneTimeAddFolderButton_clicked(self, widget):
        """ Add path in One-time backup """
        self.OneTimeView1.addFolder()
    def on_OneTimeAddFileButton_clicked(self, widget):
        """ Add file in One-time backup """
        self.OneTimeView1.addFile()
    def on_OneTimeEditButton_clicked(self, widget):
        """ Edit Path in One-time backup """
        self.OneTimeView1.editPath()
    def on_OneTimeRemoveButton_clicked(self, widget):
        """ Remove path in One-time backup """
        self.OneTimeView1.removePath()

    def on_OneTimeOptionsButton_clicked(self, widget):
        optionsDialog = OptionsDia(self.ui.options_dia)
        self.OneTimeView1._restore()
        response = optionsDialog.run()
        if response == gtk.RESPONSE_HELP:
            self.help()
            response = optionsDialog.run()
        optionsDialog.destroy()
        if response == gtk.RESPONSE_OK:
            self.OneTimeView1._save()
            

    def on_BackupSetIconview_selection_changed(self, widget):
        """ Selection changed, update QuickEdit! """
        try:
            result = self.SetView1.get_enabled()
        except Exception, e:
            errorDialog = ErrorDia(self.ui.error_dia, self.ui.ErrorDiaLabel,\
                                   str(e))
            errorDialog.run()
            errorDialog.destroy()
            return
        if result == "none":
            # none selected, disable QE
            #self.ui.QuickEditFrame.set_sensitive(False)
            #self.ui.QuickEditEnabledCheck.set_active(False)
            self.ui.edit_set1.set_sensitive(False)
            self.ui.remove_set1.set_sensitive(False)
            self.ui.duplicate_set1.set_sensitive(False)
            self.ui.backup_set_now1.set_sensitive(False)
            self.ui.restore_set1.set_sensitive(False)
            self.ui.restore_set2.set_sensitive(False)
        elif result:
            #self.ui.QuickEditFrame.set_sensitive(True)
            #self.ui.QuickEditEnabledCheck.set_active(True)
            self.ui.edit_set1.set_sensitive(True)
            self.ui.remove_set1.set_sensitive(True)
            self.ui.duplicate_set1.set_sensitive(True)
            self.ui.backup_set_now1.set_sensitive(True)
            self.ui.restore_set1.set_sensitive(True)
            self.ui.restore_set2.set_sensitive(True)
        else:
            #self.ui.QuickEditFrame.set_sensitive(True)
            #self.ui.QuickEditEnabledCheck.set_active(False)
            self.ui.edit_set1.set_sensitive(False)
            self.ui.remove_set1.set_sensitive(False)
            self.ui.duplicate_set1.set_sensitive(False)
            self.ui.backup_set_now1.set_sensitive(False)
            self.ui.restore_set1.set_sensitive(False)
            self.ui.restore_set2.set_sensitive(False)

    #def on_QuickEditEnabledCheck_toggled(self, widget):
        """ Set enabled flag when toggled """
        #if self.ui.QuickEditEnabledCheck.get_active():
        #    self.SetView1.set_enabled(True)
        #else:
        #    self.SetView1.set_enabled(False)
        #    self.on_BackupSetIconview_selection_changed(widget)

    def on_OneTimeEditOptionsButton_clicked(self, widget):
        """ Edit Options """
        optionDialog = OptionDialog(self.ui.options_dia)

    ### TAB 4: RESTORE
    def _populateDates(self, setname, box):
        """ Populate a ComboBox widget with dates of backups """
        box.get_model().clear()
        set = config.BackupSetConf(setname)
        try:
            listing = os.listdir(set.get('Options', 'Destination'))
            listing.sort()
        except OSError, e:
            errorDia = ErrorDia(\
                                self.ui.error_dia, \
                                self.ui.ErrorDiaLabel, \
                                _('Error obtaining file listing:\n%(a)s') % {'a': str(e)}\
                                )
            errorDia.run()
            errorDia.destroy()
            return
        selections = []
        for i in listing:
            if i.startswith('%s-%s-' % (_('Backup'), setname)):
                box.get_model().append([' tar'.join('-'.join(i.split('-')[2:]).split('.tar'))])
                #count = count + 1
        box.set_active(0)

    """def _checkSourcePerms(self, path):
        if utils.CheckPerms(path):
            self.ui.RestoreSourcePermissionImage.set_from_stock(gtk.STOCK_OK, gtk.ICON_SIZE_BUTTON)
            #self.ui.RestoreSourcePermissionLabel.set_text(_('You have read permissions'))
        else:
            self.ui.RestoreSourcePermissionImage.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
            #self.ui.RestoreSourcePermissionLabel.set_text(_('You do NOT have read permissions'))"""

    def _checkDestPerms(self, path):
        if utils.CheckPerms(path):
            self.ui.RestoreDestinationPermissionImage.set_from_stock(gtk.STOCK_OK, gtk.ICON_SIZE_BUTTON)
            #self.ui.RestoreDestinationPermissionLabel.set_text(_('You have read and write permissions'))
        else:
            self.ui.RestoreDestinationPermissionImage.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
            #self.ui.RestoreDestinationPermissionLabel.set_text(_('You do NOT have read and write permissions'))

    def _getPathFromCombobox(self, setname):
        """ Return the path for a given setname based on box choices """
        set = config.BackupSetConf(setname)
        dest = set.get('Options', 'Destination')
        path = '%s%s%s-%s-%s' % (dest, pathsep, _('Backup'),\
                                 self.ui.RestoreSetNameBox.get_model().get_value(\
self.ui.RestoreSetNameBox.get_active_iter(), 0), \
'.'.join(self.ui.RestoreSetDateBox.get_model().\
get_value(self.ui.RestoreSetDateBox.get_active_iter(), 0).split(' ')))
        return path

    def __checkval(self, model, path, iter, user_data):
        setname = user_data[0]
        if model.get_value(iter, 0).strip() == setname.strip():
            return True
        else:
            self.count = self.count + 1
            return False

    def _setComboboxFromName(self, setname):
        """ Set the box choices based on setname """
        self.count = 0
        self.ui.RestoreSetNameBox.get_model().foreach(self.__checkval, [setname])
        self.ui.RestoreSetNameBox.set_active(self.count)
        del(self.count)

    """def on_RestoreSourceFolderRadio_toggled(self, widget):
        if widget.get_active():
            path = self.ui.RestoreSource.get_filename()
            self._checkSourcePerms(path)"""

    #def on_RestoreSourceSetRadio_toggled(self, widget):
    #    if widget.get_active():
    #        try:
    #            path = self._getPathFromCombobox(self.ui.RestoreSetNameBox.\
    #get_model().get_value(self.ui.RestoreSetNameBox.get_active_iter(), 0))
    #            self._checkSourcePerms(path)
    #        except TypeError:
    #            # Set to no perms
    #            self.ui.RestoreSourcePermissionImage.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
    #            self.ui.RestoreSourcePermissionLabel.set_text(_('You do NOT have read permissions'))

    def on_RestoreSourceArchive_selection_changed(self, widget):
        self.ui.RestoreSourceArchiveRadio.set_active(True)
    """    try:
            path = self.ui.RestoreSourceArchive.get_filename()
            if path != None:
                self._checkSourcePerms(path)
        except IndexError:
             Set to no perms
            self.ui.RestoreSourcePermissionImage.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
            self.ui.RestoreSourcePermissionLabel.set_text(_('You have read permissions'))

    def on_RestoreSourceArchiveRadio_toggled(self, widget):
        try:
            path = self.ui.RestoreSourceArchive.get_filename()
            if path != None:
                self._checkSourcePerms(path)
        except IndexError:
            # Set to no perms
            self.ui.RestoreSourcePermissionImage.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
            self.ui.RestoreSourcePermissionLabel.set_text(_('You do NOT have read permissions'))"""


    def on_RestoreSetNameBox_changed(self, widget):
        self.ui.RestoreSourceSetRadio.set_active(True)
        self._populateDates(self.ui.RestoreSetNameBox.get_model().get_value(\
self.ui.RestoreSetNameBox.get_active_iter(), 0), self.ui.RestoreSetDateBox)
        """try:
            path = self._getPathFromCombobox(self.ui.RestoreSetNameBox.get_model()\
.get_value(self.ui.RestoreSetNameBox.get_active_iter(), 0))
            self._checkSourcePerms(path)
        except TypeError:
            # Set to no perms
            self.ui.RestoreSourcePermissionImage.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
            self.ui.RestoreSourcePermissionLabel.set_text(_('You do NOT have read permissions'))"""

    def on_RestoreSetDateBox_changed(self, widget):
        self.ui.RestoreSourceSetRadio.set_active(True)
        """path = self.ui.RestoreSource.get_filename()
        try:
            path = self._getPathFromCombobox(self.ui.RestoreSetNameBox.get_model().get_value(self.ui.RestoreSetNameBox.get_active_iter(), 0))
            self._checkSourcePerms(path)
        except TypeError:
            # Set to no perms
            self.ui.RestoreSourcePermissionImage.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
            self.ui.RestoreSourcePermissionLabel.set_text(_('You do NOT have read permissions'))"""

    def on_RestoreSource_current_folder_changed(self, widget):
        self.ui.RestoreSourceFolderRadio.set_active(True)
        #path = self.ui.RestoreSource.get_filename()
        #self._checkSourcePerms(path)

    def on_RestoreDestination_current_folder_changed(self, widget):
        path = self.ui.RestoreDestination.get_filename()
        self._checkDestPerms(path)

    def _toggleLocked(self, ignore):
        """ Toggle locking in the UI """
        for widget in   [\
self.ui.BackupSetsRadioTool,\
self.ui.OneTimeRadioTool, self.ui.RestoreRadioTool,\
self.ui.vbuttonbox1, self.ui.OneTimeVButtonBox,\
self.ui.BackupSetCancelButton, self.ui.OneTimeDestination,\
self.ui.BackupSetIconview,\
self.ui.OneTimeOptionsButton,\
self.ui.backup_set_now1, self.ui.backup_sets1,\
self.ui.one_time_backup1,\
self.ui.restore1, self.ui.frame34,\
self.ui.RestoreDestination\
]:
            try:
                ignore.index(widget)
            except ValueError:
                widget.set_sensitive(not widget.get_property('sensitive'))
        for widget in [\
self.ui.new_set1, self.ui.backup_set_now1,\
self.ui.import_sets1,\
self.ui.export_sets1, self.ui.restore_set1,\
self.ui.restore_set2, self.ui.edit_set1,\
self.ui.remove_set1, self.ui.duplicate_set1\
]:
            widget.set_sensitive(False)

    def on_RestoreStartButton_clicked(self, widget):
        prefs = config.PrefsConf()
        self.ui.MainNotebook.set_current_page(3)
        self.ui.RestoreCancelButton.set_sensitive(True)
        self.ui.RestoreStartButton.set_sensitive(False)
        self.operationInProgress = True
        if int(prefs.get('Preferences', 'ShowNotifications')) == 1:
            self.trayNotify(_('Status'), _('Starting a restore operation'), 5)
        self.ui.RestoreRadioTool.set_active(True)
        try:
            if self.ui.RestoreSourceSetRadio.get_active():
                source = self._getPathFromCombobox(self.ui.RestoreSetNameBox.get_model().get_value(self.ui.RestoreSetNameBox.get_active_iter(), 0))
            elif self.ui.RestoreSourceFolderRadio.get_active():
                source = self.ui.RestoreSource.get_filename()
            elif self.ui.RestoreSourceArchiveRadio.get_active():
                source = self.ui.RestoreSourceArchive.get_filenames()[0]
                if not source.endswith('.tar.gz') and not source.endswith('.tar'):
                    errorDialog = ErrorDia(self.ui.error_dia, self.ui.ErrorDiaLabel, \
                                           _('The file you selected is not a supported archive.'))
                    errorDialog.run()
                    errorDialog.destroy()
                    return
        except Exception, e:
            self.statusbar.newmessage(_('Please select a valid source.'))
            return
        #guiutils.busyCursor(self.ui.main)
        progressbar = ProgressBar(self.ui.RestoreProgress)
        progressbar.set_text(_('Please wait...'))
        progressbar.startPulse()
        dest = self.ui.RestoreDestination.get_filename()
        self._toggleLocked([self.ui.RestoreRadioTool, self.ui.restore1])
        try:
            self.restoreHandle = backend.Restore(source, dest, self.logger)
            self.restoreThread = guiutils.runFuncAsThread(self.restoreHandle.restore)
            while self.restoreThread.isAlive() == True:
                while gtk.events_pending():
                    gtk.main_iteration()
                time.sleep(0.01)
        except Exception, e:
            errorDia = ErrorDia(\
                                self.ui.error_dia, \
                                self.ui.ErrorDiaLabel, \
                                _('An error occurred initializing the restore process.')\
                                )
            errorDia.run()
            errorDia.destroy()
            raise
            self.logger.logmsg('DEBUG', _('An error occurred initializing the backup process:\n%s') % str(e))
        self.updateReturn = False
        progressbar.stopPulse()
        self._toggleLocked([self.ui.RestoreRadioTool, self.ui.restore1])
        for widget in [\
self.ui.new_set1,
self.ui.backup_set_now1, self.ui.import_sets1,\
self.ui.export_sets1, self.ui.restore_set1,\
self.ui.restore_set2,
]:
            widget.set_sensitive(True)
        self.ui.RestoreCancelButton.set_sensitive(False)
        self.ui.RestoreStartButton.set_sensitive(True)
        progressbar.set_text(_('Click \'Restore\' to begin'))
        #guiutils.normalCursor(self.ui.main)
        self.operationInProgress = False
        if int(prefs.get('Preferences', 'ShowNotifications')) == 1:
            self.trayNotify(_('Status'), _('Finished the restore operation'), 5)

    def on_RestoreCancelButton_clicked(self, widget):
        self.updateReturn = False
        self.restoreHandle.cancel()
        self.ui.RestoreProgress.set_text(_('Cancelling...'))
    
    """def updateFailedBackupRunsLabel(self):
        \""" Updates the failed backup label \"""
        count = self.ui.FailedBackupRunsLabel.get_text()
        self.failedruns = int(count) + 1
        self.ui.FailedBackupRunsLabel.set_text('<span color="red">%s</span>' \
            % self.errwarns)
        self.ui.FailedBackupRunsLabel.set_use_markup(True)"""
        
    def updateErrorWarnLabel(self, severity, msg, flash=True):
        """ Updates the error / warning label, flashing when applicable """
        if severity == "error" or severity == "warning":
            count = self.ui.ErrorsInLogLabel.get_text()
            self.errwarns = int(count) + 1
            self.ui.ErrorsInLogLabel.set_text('<span color="red">%s</span>' \
            % self.errwarns)
            self.ui.ErrorsInLogLabel.set_use_markup(True)
            if flash:
                trigger = 0
                while trigger < 2:
                    if gtk.events_pending():
                        gtk.main_iteration()
                    time.sleep(0.1)
                    self.ui.ErrorsInLogLabel.set_text('<span color="yellow">%i</span>'\
                                                      % self.errwarns)
                    self.ui.ErrorsInLogLabel.set_use_markup(True)
                    if gtk.events_pending():
                        gtk.main_iteration()
                    time.sleep(0.1)
                    self.ui.ErrorsInLogLabel.set_text('<span color="red">%i</span>'\
                                                      % self.errwarns)
                    self.ui.ErrorsInLogLabel.set_use_markup(True)
                    trigger += 1
            
    ### TAB 5: LOGGER
    def on_LogViewerClearButton_clicked(self, widget):
        """ Clear the log """
        def clearlog():
            """ Does the actual log clearing """
            self.loghandler.clear()
            self.ui.ErrorsInLogLabel.set_text('0')
            self.ui.FailedBackupRunsLabel.set_text('0')
            self.logger.logmsg('INFO', _('Log cleared'))
        prefs = config.PrefsConf()
        if int(prefs.get('Preferences', 'DontShowMe_ClearLog')) == 1:
            return clearlog()
        dia = ConfirmDia(self.ui.confirm_dia, self.ui.ConfirmDiaLabel, \
_('This will permanently remove all log entries. Are you sure you want to continue?'), dontShowMe=self.ui.ConfirmDiaDontAgainCheck)
        response = dia.run()
        if response == gtk.RESPONSE_YES:
            if self.ui.ConfirmDiaDontAgainCheck.get_active():
                prefs.set('Preferences', 'DontShowMe_ClearLog', 1)
            else:
                prefs.set('Preferences', 'DontShowMe_ClearLog', 0)
            clearlog()
        dia.destroy()
    
    def on_LogViewerSaveToFileButton_clicked(self, widget):
        """ Save log to file """
        saveDialog = SaveDia(self.ui.save_dia)
        response = saveDialog.run()
        if response == gtk.RESPONSE_OK:
            path = saveDialog.get_filename()[0]
            saveto = open(path, 'w')
            readfrom = open(logloc, 'r')
            saveto.write(readfrom.read())
            readfrom.close()
            saveto.close()
            self.logger.logmsg('DEBUG', _('Log saved to file \'%s\'') % path)
        saveDialog.destroy()

    ### EXPORT_DIA ###

    def on_ExportFileChooserButton_current_folder_changed(self, widget):
        self.ExportView1._checkDestPerms(widget.get_filename(),self.ui.ExportPermissionImage)

